=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Copyright © 2008 Fredo6 - Designed and written Dec 2008 by Fredo6
#
# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:   Fredo6_!LibFredo6.rb
# Original Date	:   20 Aug 2008 - version 3.0
# Type			:   Ruby Library
# Description	:   Top loading module for all Library utilities of Fredo6's scripts
# Usage			:   See Tutorial and Quick Ref Card in PDF format
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

unless defined?(LibFredo6)

require 'sketchup.rb'
require 'extensions.rb'
begin
	require 'LibTraductor.rb' if Sketchup.find_support_file('LibTraductor.rb', "Plugins")	#Compatibility
	require 'TOS_Dir_13/LibTraductor_20.rb' if Sketchup.find_support_file('TOS_Dir_13/LibTraductor_20.rb', "Plugins")	#Compatibility
rescue Exception => e
end

module LibFredo6

@@time_start = Time.now
if defined?(Sketchup.temp_dir)
	@@tmp_dir = Sketchup.temp_dir
else	
	if RUBY_PLATFORM =~ /darwin/i
		@@tmp_dir = ENV["TMPDIR"]
		@@tmp_dir = "/tmp" unless FileTest.directory?(@@tmp_dir)
	else
		@@tmp_dir = ENV["TEMP"].gsub(/\\/, '/')
	end	
end

#Trace logging
def LibFredo6.tmpdir ; @@tmp_dir ; end
def LibFredo6.log_file ; @logfile_path ; end
def LibFredo6.rootlog ; "LibFredo6Trace_#{Sketchup.version.to_i}_*.txt" ; end
def self.mask_dir(s) ; (s.class == String && s =~ /(\.rb.*:\d+:)/) ? File.basename($`) + $1 + " " + $' : s ; end
def LibFredo6.sagwarum(*args) ; LibFredo6.sag(*args) ; end
def LibFredo6.sag(*args) ; LibFredo6.sag(*args) ; end

def LibFredo6.log(*args)
	t0 = args.find { |a| a.class == Time }
	args = args.find_all { |a| a }
	if args[0].class == Time
		t0 = args[0]
		args = args[1..-1]
	else	
		t0 = Time.now
	end	
	unless @logfile_handle
		@code = ">>>> "
		tmpdir = @@tmp_dir
		@logfile_path = File.join tmpdir, LibFredo6.rootlog.sub('*', "#{(@@time_start.to_f*1000).to_i}")
		@logfile_handle = File.open(@logfile_path, "w")
		return false unless @logfile_handle
		LibFredo6.log "DATE / TIME: #{@@time_start}", "SKETCHUP VERSION: #{Sketchup.version}", "RUBY_PLATFORM: #{RUBY_PLATFORM}",
		              "LOG FILE: #{File.basename(@logfile_path)}"
	end
	lines = caller
	line = (caller[0] =~ /sagwarum/) ? lines[1] : lines[0]
	@logfile_handle.puts "#{@code}#{t0.to_f};#{mask_dir line}"
	largs2 = args.collect { |s| mask_dir(s) }
	@logfile_handle.puts *largs2
	@logfile_handle.puts "\n"
	@logfile_handle.flush
	true
end

#Show a message box for error
def LibFredo6.log_messagebox(*args)
	text = args.join("\n") + "\n\n" + caller[0]
	UI.messagebox text
end

#Debug Logging
def LibFredo6.debug_file ; @debugfile_path ; end
def LibFredo6.rootdebug ; "LibFredo6Debug_#{Sketchup.version.to_i}_*.txt" ; end
def LibFredo6.debug2(*args) ; puts LibFredo6.debug(*args) ; end

def LibFredo6.debug(*args)
	unless @debugfile_handle
		tmpdir = @@tmp_dir
		@debugfile_path = File.join tmpdir, LibFredo6.rootdebug.sub(/\*/, "#{(@@time_start.to_f*1000).to_i}")
		@debugfile_handle = File.open(@debugfile_path, "w")
		return false unless @debugfile_handle
		LibFredo6.debug "DATE / TIME: #{@@time_start}", "SKETCHUP VERSION: #{Sketchup.version}", "RUBY_PLATFORM: #{RUBY_PLATFORM}",
		              "LOG FILE: #{File.basename(@debugfile_path)}", " "
	end
	args[0] = ["\n=========== time = #{Time.now.to_f} =============="] if args.empty? || args[0] == ""
	@debugfile_handle.puts *args
	@debugfile_handle.flush
	args
end

#Register the latest version of LibFredo6 family 
def LibFredo6.startup
	@finished_loading_cycle = false
	LibFredo6.log "LibFredo6: Starting up Fredo6_!LibFredo6.rb"
	
	#Locating LibFredo6
	file__ = __FILE__
	file__ = file__.force_encoding("UTF-8") if defined?(Encoding)
	file__ = file__.gsub(/\\/, '/')
	@lib_fredo_sudir = File.dirname file__
	
	#Compatibility with previous naming conventions
	LibFredo6.manage_compatibility
	
	#Loop of subfolders in the SU directory
	folder6 = nil
	Dir[File.join(@lib_fredo_sudir, '*/')].each do |folder|
		folder_name = File.basename folder
		if folder_name == "Fredo6_!LibFredo6"
			folder6 = folder_name
			break
		elsif folder_name =~ /\AFredo6_!LibFredo6_(\d\d)\Z/
			folder6 = folder_name
		end	
	end
	
	#LibFredo6 subfolder not found
	unless folder6
		text = "LibFredo6: Folder LibFredo6 not found in directory #{@lib_fredo_sudir}"
		LibFredo6.log text
		UI.messagebox text
		return
	end
	
	@lib_fredo_path = File.join @lib_fredo_sudir, folder6
	
	#Checking the subfolder is correct
	lib_pattern = "Lib6Core*.rb"
	core_rb = Dir[File.join(@lib_fredo_path, lib_pattern)].last
	
	unless core_rb
		text = "LibFredo6: Folder #{folder6} is not valid - Please install correctly"
		LibFredo6.log text
		UI.messagebox text
		return
	end

	#Directory for APP data and management of retro-compatibility
	LibFredo6.dir_appdata
	
	#Compatibility for Documentation. Move files in the Plugins directory
	begin
		old_dir = File.join @lib_fredo_sudir, "DOCUMENTATION_Dir"
		if FileTest.directory?(old_dir)
			Dir[File.join(old_dir, "*.*")].each do |f|
				File.rename f, File.join(@lib_fredo_sudir, File.basename(f))
			end	
		end
	rescue
	end

	#Loading LibFredo6
	@lib_fredo_folder = File.basename @lib_fredo_path
	LibFredo6.log "LibFredo6: Beginning of loading Cycle***+"
	
	#Loading the minimum routines to enable the LibFredo6 environment (file Lib6Core_xx.rb)
	@core_loaded = File.basename(core_rb)
	rbfile = File.join(@lib_fredo_folder, @core_loaded)
	begin
		require core_rb
		LibFredo6.log "LibFredo6 Core: #{rbfile} successfully loaded"
	rescue Exception => e
		LibFredo6.log "!LibFredo6 Core: ERROR loading #{rbfile}", "#{LibFredo6.exception_message(e)}"
		return
	end
	
	#Registering the rest of the LibFredo6 library (but not loading it)
	Traductor::Plugin.new.load_from_config "LibFredo6", @lib_fredo_folder

	LibFredo6.log "LibFredo6: End of loading Cycle***-"
	UI.start_timer(1, false) { LibFredo6.after_startup ; @finished_loading_cycle = true }
end

#Manage the compatibility with files belonging to the old naming conventions
def LibFredo6.manage_compatibility
	#For compatibility: avoid overriding by the very old library
	$".push "LibTraductor.rb", "libtraductor.rb"
	
	#Make sure the old LibFredo6.rb will not load
	$".push "LibFredo6.rb"
	
	#For FredoScale 2.4 and RoundCorner 2.4, skip the top file when they contains an explicit test on "libFredo6.rb"
	["Fredo6_RoundCorner.rb", "Fredo6_FredoScale.rb"].each do |fname|
		file = File.join @lib_fredo_sudir, fname
		next unless FileTest.exist?(file)
		old = false
		IO.foreach(file) do |line| 
			if line =~ /"LibFredo6\.rb"/
				$".push fname
				old = true
				break
			end	
			break if old
		end
	end
	
	#Compatibility with EWH
	begin ; SketchupExtension.new "toto", "toto.rb" ; rescue ; end
end

#Build a message for exception tracing
def LibFredo6.exception_message(e)
	e.backtrace[0] =~ /(\d+):in\s(.*)'/
	file = $`
	(file) ? "#{e.message} [in #{File.basename($`)} - ##{$2}(): line #{$1}]" : "#{e.message} -  #{e.backtrace[0]}"
end

#Task to execute right after startup
def LibFredo6.after_startup
	Traductor::TraceLog.purge if defined?(Traductor::TraceLog.purge)
	Traductor::DebugLog.purge if defined?(Traductor::DebugLog.purge)
	Traductor::Plugin.signal_error_in_loading if defined?(Traductor::Plugin.signal_error_in_loading)
	Traductor::Upgrade.time_for_check? if defined?(Traductor::Upgrade.time_for_check?)

	#Login the version if not already done
	Traductor::AllPlugins.scf_version_log
end

#Register a Plugin from a configuration file
def LibFredo6.register_plugin(rootname, folder, plugin_name=nil)
	#obsolete - left because called by Z_Loader__ files
end

#Manage the creation and compatibility for App data directories
#Currently, where Default Parameters and Documentation is stored
def LibFredo6.dir_appdata
	#Determining the location for root		
	if RUBY_PLATFORM =~ /darwin/i
		dirtop = File.expand_path("~/Library/Application Support")
		dirtop = @@tmp_dir unless FileTest.directory?(dirtop) && File.writable?(dirtop)
		hdot = '.'
	else
		hdot = ''
		dirtop = File.dirname @@tmp_dir
		a = (ENV["LOCALAPPDATA"]) ? nil : ENV["APPDATA"]
		dirtop = File.join dirtop, File.basename(a) if a	#for Windows XP
		@dir_appdata = File.join dirtop, "LIBFREDO6_DATA_Dir"
	end	
	dirtop = @@tmp_dir unless FileTest.directory?(dirtop) && File.writable?(dirtop)
	
	#Name of the subfolders
	@dir_appdata = File.join dirtop, hdot + "LIBFREDO6_DATA_Dir"
	@dir_defparam = File.join @dir_appdata, "DefaultParameters"
	new_install = !(FileTest.directory?(@dir_defparam))
	
	LibFredo6.log "Temp Files directory: #{@@tmp_dir}"
	LibFredo6.log "Default Parameters folder: #{@dir_defparam}"
	
	#Create the top app data directory if it does not exist
	begin
		Dir.mkdir @dir_appdata unless FileTest.directory?(@dir_appdata)
	rescue
		@dir_appdata = @@tmp_dir
	end	
	begin
		Dir.mkdir @dir_defparam unless FileTest.directory?(@dir_defparam)
	rescue
		@dir_defparam = @@tmp_dir
		UI.messagebox "LibFredo6: Problem of installation in creation of Data directories in #{dirtop}"
		return
	end

	#Compatibility for Default Parameters - Copying the previous files if any and once
	old_dir = File.join @lib_fredo_sudir, "DEFPARAM_Dir"
	if new_install && FileTest.directory?(old_dir)
		Dir[File.join(old_dir, "*.def")].each do |f|
			lines = IO.readlines(f)
			File.open(File.join(@dir_defparam, File.basename(f)), "w") { |ff| lines.each { |text| ff.puts text } }
			LibFredo6.log "LibFredo6 Core: Transfering parameters from #{File.basename(f)}"
		end	
	end
end

#Return module variables for LibFredo6
def LibFredo6.sudir ; @lib_fredo_sudir ; end
def LibFredo6.path ; @lib_fredo_path ; end
def LibFredo6.folder ; @lib_fredo_folder ; end
def LibFredo6.core_loaded ; @core_loaded ; end
def LibFredo6.appdata_dir ;@dir_appdata ; end
def LibFredo6.defparam_dir ; @dir_defparam ; end
def LibFredo6.finished_loading_cycle ; @finished_loading_cycle ; end
def LibFredo6.finished_loading_cycle=(a) ; @finished_loading_cycle = a ; end

#-------------------------------------------------------------------------------------	
#Always Loading the Lib6Core file (minimum code)
#-------------------------------------------------------------------------------------	

LibFredo6.startup	
	
end	#Module LibFredo6

else
	UI.messagebox "LibFredo6 will be updated at next startup of Sketchup"
end	#Defined? LibFredo6